#include "marbles.h"

/***********************************************************/
/* SUPPORT FUNCTIONS.                                      */
/***********************************************************/
/****************************************/
/* ROUND.                               */
/****************************************/
int ROUND(double x) {
   return (int)floor(0.5 + x);
}

/************************************************************/
/* INITIALIZATION.                                          */
/************************************************************/
/****************************************/
/* Textures.                            */
/****************************************/
void TEXTURE_MODEL(void) {
int w, h, c;
uchar *data;
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
   // Load texture map for tie marbles.
   data = PGM_FILE_READ_1D("./gray.ppm", &w, &h, &c);
   glGenTextures(1, &TieMap);
   glBindTexture(GL_TEXTURE_2D, TieMap);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
   free(data);
   // Load texture map for red marbles.
   data = PGM_FILE_READ_1D("./red.ppm", &w, &h, &c);
   glGenTextures(1, &RedMap);
   glBindTexture(GL_TEXTURE_2D, RedMap);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
   free(data);
   // Load texture map for blue marbles.
   data = PGM_FILE_READ_1D("./blue.ppm", &w, &h, &c);
   glGenTextures(1, &BlueMap);
   glBindTexture(GL_TEXTURE_2D, BlueMap);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
   free(data);
   // Load texture map for grass.
   data = PGM_FILE_READ_1D("./felt.ppm", &w, &h, &c);
   glGenTextures(1, &FeltMap);
   glBindTexture(GL_TEXTURE_2D, FeltMap);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_REPEAT); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
   free(data);
   // Load texture map for wood.
   data = PGM_FILE_READ_1D("./wood.ppm", &w, &h, &c);
   glGenTextures(1, &WoodMap);
   glBindTexture(GL_TEXTURE_2D, WoodMap);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_REPEAT); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
   free(data);
}

/****************************************/
/* Lighting model.                      */
/****************************************/
void LIGHT_MODEL(void) {
GLfloat ambient[]    = {0.1, 0.1, 0.1, 1.0}; 
GLfloat diffuseD[]   = {0.7, 0.7, 0.7, 0.0};
GLfloat PositionD[]  = {lightX, lightY, 200.0, 0.0};
   glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR); 
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   // Ambient "light source".
   glColorMaterial(GL_FRONT, GL_AMBIENT);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   // Diffuse light source.
   glColorMaterial(GL_FRONT, GL_DIFFUSE);
   glLightfv(GL_LIGHT1, GL_DIFFUSE,  diffuseD);
   glLightfv(GL_LIGHT1, GL_POSITION, PositionD);
}

/****************************************/
/* Create marbles.                      */
/****************************************/
void CREATE_MARBLES(void) {
int    i;
double r, o;
double xR[] = {445.0, 445.0, 445.0, 445.0, 445.0, 445.0, 0.0};
double yR[] = {280.0, 310.0, 340.0, 390.0, 420.0, 450.0, 0.0};
double xB[] = {355.0, 385.0, 415.0, 475.0, 505.0, 535.0, 0.0};
double yB[] = {365.0, 365.0, 365.0, 365.0, 365.0, 365.0, 0.0};
   // Create red marbles at fixed locations.
   for (i = 0; i < 7; i++) {
       Red[i].x      	  = xR[i];
       Red[i].y      	  = yR[i];
       Red[i].z      	  = 40.0;
       Red[i].radius      = 10.0;
       Red[i].velocityX   = 0.0;
       Red[i].velocityY   = 0.0;
       Red[i].restitution = 0.658;				// Glass
       Red[i].mass        = 1.0;
   }
   // Create 1 red marble at random location.
   r = (double)(200 + rand()%90);
   o = ((double)(rand()%360) / 360.0) * 2 * 3.14159;
   Red[6].x = 445.0 + r*sin(o);
   Red[6].y = 365.0 + r*cos(o);
   // Create Blue marbles at fixed locations.
   for (i = 0; i < 7; i++) {
       Blue[i].x      	  = xB[i];
       Blue[i].y      	  = yB[i];
       Blue[i].z      	  = 40.0;
       Blue[i].radius     = 10.0;
       Blue[i].velocityX  = 0.0;
       Blue[i].velocityY  = 0.0;
       Blue[i].restitution= 0.658;				// Glass
       Blue[i].mass       = 1.0;
   }
   // Create 1 blue marble at random location.
   r = (double)(200 + rand()%90);
   o = o + 3.14159;
   Blue[6].x = 445.0 + r*sin(o);
   Blue[6].y = 365.0 + r*cos(o);
}

/************************************************************/
/* GAME BOARD DISPLAY.                                      */
/************************************************************/
/****************************************/
/* Main window display.                 */
/****************************************/
void DRAW_MENU(void) {
char label[100];
   glutSetWindow(window);
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glClear(GL_COLOR_BUFFER_BIT); 
   // Draw screen panel.
   glColor3f(0.6, 0.6, 0.6);
   glBegin(GL_QUADS);
     glVertex2i(       0,         0);
     glVertex2i(winWidth,         0);
     glVertex2i(winWidth, winHeight);
     glVertex2i(       0, winHeight);
   glEnd();
   // Draw buttons.
   RULES_BUTTON.DRAW();
   START_BUTTON.DRAW();
   ABOUT_BUTTON.DRAW();
   EXIT_BUTTON.DRAW_HILITE(0.7, 0.0, 0.0);
   MAIN_WINDOW.DRAW();
   glFlush();
   glutSwapBuffers();
}

/****************************************/
/* Draws game board wood border.        */
/****************************************/
void DRAW_BORDER(void) {
   glutSetWindow(vuport);
   // Draw box.
   glLoadIdentity();
   glEnable(GL_TEXTURE_2D);  
   glBindTexture(GL_TEXTURE_2D, WoodMap);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
   // Left side.
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  1.0); 
     glVertex3f(  0.0, 10.0,  60.0);	// Bottom Left 
     glTexCoord2f(1.0,  1.0); 
     glVertex3f( 10.0, 10.0,  60.0);	// Bottom Right 
     glTexCoord2f(1.0,  0.0); 
     glVertex3f( 10.0,720.0,  60.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f(  0.0,720.0,  60.0);	// Top Left 
   glEnd();
   // Bottom side.
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  1.0); 
     glVertex3f(  0.0,  0.0,  60.0);	// Bottom Left 
     glTexCoord2f(1.0,  1.0); 
     glVertex3f(890.0,  0.0,  60.0);	// Bottom Right 
     glTexCoord2f(1.0,  0.0); 
     glVertex3f(890.0, 10.0,  60.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f(  0.0, 10.0,  60.0);	// Top Left 
   glEnd();
   // Right side.
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  1.0); 
     glVertex3f(880.0, 10.0,  60.0);	// Bottom Left 
     glTexCoord2f(1.0,  1.0); 
     glVertex3f(890.0, 10.0,  60.0);	// Bottom Right 
     glTexCoord2f(1.0,  0.0); 
     glVertex3f(890.0,720.0,  60.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f(880.0,720.0,  60.0);	// Top Left 
   glEnd();
   // Top side.
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  1.0); 
     glVertex3f(  0.0,720.0,  60.0);	// Bottom Left 
     glTexCoord2f(1.0,  1.0); 
     glVertex3f(890.0,720.0,  60.0);	// Bottom Right 
     glTexCoord2f(1.0,  0.0); 
     glVertex3f(890.0,730.0,  60.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f(  0.0,730.0,  60.0);	// Top Left 
   glEnd();
   glDisable(GL_TEXTURE_2D); 
}

/****************************************/
/* Draws game board fake grass.         */
/****************************************/
void DRAW_GRASS(void) {
   glutSetWindow(vuport);
   // Draw grass.
   glLoadIdentity();
   glEnable(GL_TEXTURE_2D);  
   glBindTexture(GL_TEXTURE_2D, FeltMap);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  4.0); 
     glVertex3f( 10.0, 10.0,  0.0);	// Bottom Left 
     glTexCoord2f(4.0,  4.0); 
     glVertex3f(880.0, 10.0,  0.0);	// Bottom Right 
     glTexCoord2f(4.0,  0.0); 
     glVertex3f(880.0,720.0,  0.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f( 10.0,720.0,  0.0);	// Top Left 
   glEnd();
   glDisable(GL_TEXTURE_2D); 
}

/****************************************/
/* Draws game board marbles.            */
/****************************************/
void DRAW_MARBLES(void) {
int i;
int x, y;
GLfloat x1, x2;
GLfloat y1, y2;
double  angle1, angle2;
   glutSetWindow(vuport);
   // Draw game ring.
   glLoadIdentity();
   glTranslatef(centerX, centerY, 0);
   glColor3ub(250, 250, 250);
   angle1 = 0.0;
   for (i = 0; i < 100; i++) {
       angle2 = (pi * (float)i) / 45.0;
       x1 = 300.0 * cos(angle1);	// vertix 2
       y1 = 300.0 * sin(angle1);
       x2 = 300.0 * cos(angle2);	// vertix 3
       y2 = 300.0 * sin(angle2);
       glBegin(GL_LINES);
         glVertex3f(x1, y1, 10);
         glVertex3f(x2, y2, 10);
       glEnd();
       angle1 = angle2;
   }
   // Draw marble shadows.
   glLoadIdentity();
   glColor3ub(0, 0, 0);
   for (i = 0; i < 7; i++) {
      glPushMatrix();
      glTranslatef(Red[i].x+shadow, Red[i].y-shadow, 20.0);		// Red marble shadows.
      glutSolidSphere(Red[i].radius, 100, 100);
      glPopMatrix();
      glPushMatrix();
      glTranslatef(Blue[i].x+shadow, Blue[i].y-shadow, 20.0);	// Blue marble shadows.
      glutSolidSphere(Blue[i].radius, 100, 100);
      glPopMatrix();
   }
   // Enable light model.
   LIGHT_MODEL();
   glEnable(GL_NORMALIZE);
   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_LIGHT1);
   // Draw marbles.
   glLoadIdentity();
   for (i = 0; i < 7; i++) {
      glPushMatrix();
      glColor3ub(255, 0, 0);
      glTranslatef(Red[i].x, Red[i].y, Red[i].z);			   // Red marbles.
      glutSolidSphere(Red[i].radius, 100, 100);
      glPopMatrix();
      glPushMatrix();
      glColor3ub(0, 0, 255);
      glTranslatef(Blue[i].x, Blue[i].y, Blue[i].z);			// Blue marbles.
      glutSolidSphere(Blue[i].radius, 100, 100);
      glPopMatrix();
   }
   glDisable(GL_LIGHTING);
}

/****************************************/
/* Draws game turn.                     */
/****************************************/
void DRAW_TURN(int turn) {
   glutSetWindow(vuport);
   if (turn == red)  TEXT_8_X_13_WHITE(20, 700, "Red turn");
   if (turn == blue) TEXT_8_X_13_WHITE(20, 700, "Blue turn");
}

/****************************************/
/* Draws game everything for game board.*/
/****************************************/
void DRAW_GAME(void) {
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   DRAW_BORDER();
   DRAW_GRASS();
   DRAW_MARBLES();
   DRAW_TURN(turn);
   glFlush();
   glutSwapBuffers();
}

/****************************************/
/* Displays rules.                      */
/****************************************/
void DISPLAY_RULES(void) {
   glutSetWindow(vuport);
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glClearColor(1.0, 1.0, 1.0, 1.0);
   glClear(GL_COLOR_BUFFER_BIT);
   // Display rules.
   TEXT_8_X_13_BLACK(20, 700, "1) Turns alternate between BLUE and RED players.");
   TEXT_8_X_13_BLACK(20, 680, "2) Players can only shoot their own marbles.");
   TEXT_8_X_13_BLACK(20, 660, "3) First player to get all of their marbles out of the ring wins.");
   TEXT_8_X_13_BLACK(20, 640, "4) Marbles outside of the ring can be used to shoot marbles back into the ring.");
   TEXT_8_X_13_BLACK(20, 620, "5) To shoot:");
   TEXT_8_X_13_BLACK(20, 600, " * Use the mouse to select any of your marbles by clicking the left");
   TEXT_8_X_13_BLACK(20, 580, "   mouse button while the curser is on the marble you want to shoot.");
   TEXT_8_X_13_BLACK(20, 560, " * Drag the mouse indicating the direction and velocity you want to");
   TEXT_8_X_13_BLACK(20, 540, "   use for the shot. The longer the line the greater the velocity.");
   TEXT_8_X_13_BLACK(20, 520, " * Release the mouse button.");
   TEXT_8_X_13_BLACK(20, 400, "*) Click START to start game.");
   glFlush();
   glutSwapBuffers();
}

/****************************************/
/* Displays about.                      */
/****************************************/
void DISPLAY_ABOUT(void) {
   glutSetWindow(vuport);
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glClearColor(1.0, 1.0, 1.0, 1.0);
   glClear(GL_COLOR_BUFFER_BIT);
   // Display rules.
   TEXT_8_X_13_BLACK(20, 700, "Original Code: Ron Marsh");
   TEXT_8_X_13_BLACK(20, 680, "Revision Code: Tyler Welander, Jay Peterson, Jordan Hoffman, and Jacob Johnson");
   TEXT_8_X_13_BLACK(20, 660, "Changes: *Added marble movement");
   TEXT_8_X_13_BLACK(20, 640, "         *Detect win state");
   TEXT_8_X_13_BLACK(20, 620, "         *Detect when marbles leave circle");
   TEXT_8_X_13_BLACK(20, 600, "         *Added about button");
   TEXT_8_X_13_BLACK(20, 580, "         *Added game end window");
   TEXT_8_X_13_BLACK(20, 560, "         *Added automatic restart");
   glFlush();
   glutSwapBuffers();
}

/****************************************/
/* Displays winner.                     */
/****************************************/
void DISPLAY_WINNER(int winner) {
   glutSetWindow(vuport);
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glClearColor(1.0, 1.0, 1.0, 1.0);
   glClear(GL_COLOR_BUFFER_BIT);
   glEnable(GL_TEXTURE_2D);  
   if (winner ==  red) glBindTexture(GL_TEXTURE_2D, RedMap);
   if (winner == blue) glBindTexture(GL_TEXTURE_2D, BlueMap);
   if (winner ==  tie) glBindTexture(GL_TEXTURE_2D, TieMap);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
   glLoadIdentity();
   glBegin(GL_QUADS);
     glTexCoord2f(0.0,  1.0); 
     glVertex3f(  0.0,  0.0,  0.0);	// Bottom Left 
     glTexCoord2f(1.0,  1.0); 
     glVertex3f(890.0,  0.0,  0.0);	// Bottom Right 
     glTexCoord2f(1.0,  0.0); 
     glVertex3f(890.0,730.0,  0.0);	// Top Right
     glTexCoord2f(0.0,  0.0); 
     glVertex3f(  0.0,730.0,  0.0);	// Top Left 
   glEnd();
   glDisable(GL_TEXTURE_2D); 
   glClearColor(0.0, 0.0, 0.0, 0.0);
   if (winner ==  red) TEXT_HELVETICA_18(centerX-36, centerY, "RED WINS");
   if (winner == blue) TEXT_HELVETICA_18(centerX-40, centerY, "BLUE WINS");
   if (winner ==  tie) TEXT_HELVETICA_18(centerX-18, centerY, "TIE");
   glFlush();
   glutSwapBuffers();
}

/************************************************************/
/* PHYSICS ENGINE.                                          */
/************************************************************/
/****************************************/
/* Resolve collision.    		        */
/****************************************/
void RESOLVE_COLLISION(struct _marble *marble1, struct _marble *marble2) {
double d, dx, dy;
double vectorX, vectorY;
double velocityN, velocityX, velocityY;
double restitution, impulse;
  // Collision normal vector
  dx = (marble1->x - marble2->x);
  dy = (marble1->y - marble2->y);
  d  = sqrt(dx*dx + dy*dy);
  vectorX = dx / d;
  vectorY = dy / d;
  // Calculate relative velocity
  velocityX = (marble1->velocityX - marble2->velocityX);
  velocityY = (marble1->velocityY - marble2->velocityY);
  // Calculate relative velocity in terms of the normal
  velocityN = (velocityX * vectorX)+(velocityY * vectorY);	// Dot product.
  if (velocityN > 0) return;								// Do not resolve if marbles are separating.
  // Calculate restitution
  if (marble1->restitution < marble2->restitution) restitution = marble1->restitution;
                                              else restitution = marble2->restitution;
  // Calculate impulse scalar
  impulse = -(1.0 + restitution) * velocityN / ((1.0 / marble1->mass) + (1.0 / marble2->mass));
  // Apply impulse
  if (marble1->mass < marble2->mass) {
     marble1->velocityX -= ((impulse * vectorX) / marble1->mass);
     marble1->velocityY -= ((impulse * vectorY) / marble1->mass);
     marble2->velocityX += ((impulse * vectorX) / marble2->mass);
     marble2->velocityY += ((impulse * vectorY) / marble2->mass);
  } else {    
     marble1->velocityX += ((impulse * vectorX) / marble1->mass);
     marble1->velocityY += ((impulse * vectorY) / marble1->mass);
     marble2->velocityX -= ((impulse * vectorX) / marble2->mass);
     marble2->velocityY -= ((impulse * vectorY) / marble2->mass);
  }
}

/****************************************/
/* Collision test - marble on marble.   */
/****************************************/
bool MARBLE_COLLISION(struct _marble marble1, struct _marble marble2) {
double r, d, dx, dy;
  r  = (marble1.radius + marble2.radius + 5.0);
  dx = (marble1.x - marble2.x);
  dy = (marble1.y - marble2.y);
  d  = sqrt(dx*dx + dy*dy);
  if (r >= d) return 1;
         else return 0;
}

/****************************************/
/* Collision test - marble on wall.     */
/****************************************/
void WALL_COLLISION(struct _marble *marble) {
   if (((marble->x - marble->radius) <=  10.0) && (marble->velocityX < 0.0)) marble->velocityX *= -1.0;
   if (((marble->x + marble->radius) >= 880.0) && (marble->velocityX > 0.0)) marble->velocityX *= -1.0;
   if (((marble->y - marble->radius) <=  10.0) && (marble->velocityY < 0.0)) marble->velocityY *= -1.0;
   if (((marble->y + marble->radius) >= 720.0) && (marble->velocityY > 0.0)) marble->velocityY *= -1.0;
}

/************************************************************/
/* THE GAME ENGINE.                                         */
/************************************************************/
/****************************************/
/* Animates objects.                    */
/****************************************/
void GET_INITIAL_VELOCITY(int mouseX1, int mouseX2, int mouseY1, int mouseY2) {
   if (gROK != -1) {
      Red[gROK].velocityX = (double)(mouseX2 - mouseX1) / 15.0;
      Red[gROK].velocityY = (double)(mouseY2 - mouseY1) / 15.0;
   } else if (gBOK != -1) {
      Blue[gBOK].velocityX = (double)(mouseX2 - mouseX1) / 15.0;
      Blue[gBOK].velocityY = (double)(mouseY2 - mouseY1) / 15.0;
   }
}

/****************************************/
/* Test for a winner.                   */
/****************************************/
void TEST_FOR_WINNER(void) {
int i;
int dx, dy;
   // Count number of marbles outside of ring.
   for (i = 0; i < 7; i++) {
       dx = ROUND((Red[i].x-centerX)*(Red[i].x-centerX));
       dy = ROUND((Red[i].y-centerY)*(Red[i].y-centerY));
       if ((dx + dy) > 90000) Red[i].out = true;
                         else Red[i].out = false;
       dx = ROUND((Blue[i].x-centerX)*(Blue[i].x-centerX));
       dy = ROUND((Blue[i].y-centerY)*(Blue[i].y-centerY));
       if ((dx + dy) > 90000) Blue[i].out = true;
                         else Blue[i].out = false;
   }
   // Determine if we have a winner.
   redOut = 0;
   blueOut = 0;
   for (i = 0; i < 7; i++) {
       if (Red[i].out  == true) redOut++;
       if (Blue[i].out == true) blueOut++;
   }
   printf("RED marbles out:  %i\n", redOut);
   printf("BLUE marbles out: %i\n", blueOut);
   if (redOut == 7 && blueOut == 7) {
      DISPLAY_WINNER(tie);
      restart = true;
      sleep(15);
   } else if (redOut == 7) {
      DISPLAY_WINNER(red);
      sleep(15);
      restart = true;
   } else if (blueOut == 7) {
      DISPLAY_WINNER(blue);
      restart = true;
      sleep(15);
   }
}

/****************************************/
/* Animates objects.                    */
/****************************************/
void GAME_ANIMATE(void) {
int i, j;
//int dx, dy;
double vx, vy, velocity;
int doneMoving = 0;
   // Move marbles.
   while (doneMoving != 14) {
      doneMoving = 0;
      // Update velocities
      for (i = 0; i < 7; i++) {
          Red[i].x  += Red[i].velocityX;
          Red[i].y  += Red[i].velocityY;
          Blue[i].x += Blue[i].velocityX;
          Blue[i].y += Blue[i].velocityY;
      }
      // Collision tests
      for (i = 0; i < 7; i++) {
          WALL_COLLISION(&Red[i]);
          WALL_COLLISION(&Blue[i]);
          for (j = 0; j < 7; j++) {
              if (MARBLE_COLLISION(Red[i],  Blue[j])) RESOLVE_COLLISION(&Red[i],  &Blue[j]);
              if (j == i) continue;
              if (MARBLE_COLLISION(Red[i],  Red[j]))  RESOLVE_COLLISION(&Red[i],  &Red[j]);
              if (MARBLE_COLLISION(Blue[i], Blue[j])) RESOLVE_COLLISION(&Blue[i], &Blue[j]);
          }
      }
      // Delay
      usleep(delay);
      DRAW_GAME();
      // Marble motion.
      for (i = 0; i < 7; i++) {
          // Red motion.
          vx = Red[i].velocityX*Red[i].velocityX;
          vy = Red[i].velocityY*Red[i].velocityY;
          if ((vx + vy) > 0.01) {
             Red[i].velocityX /= 1.05;
             Red[i].velocityY /= 1.05;
          } else {
		    Red[i].velocityX = 0.0;
		    Red[i].velocityY = 0.0;
		    doneMoving++;
		  }
          // Blue motion.
          vx = Blue[i].velocityX*Blue[i].velocityX;
          vy = Blue[i].velocityY*Blue[i].velocityY;
          if ((vx + vy) > 0.01) {
             Blue[i].velocityX /= 1.05;
             Blue[i].velocityY /= 1.05;
          } else {
		  	 Blue[i].velocityX = 0.0;
		  	 Blue[i].velocityY = 0.0;
			 doneMoving++;
          }
      }
   }
   TEST_FOR_WINNER();
   // Change player.
   if (turn == blue) turn = red;
                else turn = blue;
   // Update game.
   DRAW_GAME();
}

/************************************************************/
/* CALLBACK FUNTIONS.                                       */
/************************************************************/
/****************************************/
/* Mouse on marble test.                */
/****************************************/
bool MARBLE_SELECTED(struct _marble marble, int mouseX, int mouseY) {
double d, dx, dy;
   dx = (marble.x-mouseX);
   dy = (marble.y-mouseY);
   d = sqrt(dx*dx + dy*dy);
   if (marble.radius >= d) return 1;
                      else return 0;
}

/****************************************/
/* Mouse control.                       */
/****************************************/
void myMouse0(int button, int state, int x, int y) {
   glutSetWindow(window);
   y = (winHeight - y);
   /*********************/	
   /* Button controls.	*/
   /*********************/	
   if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
      glFinish();
      // Rules button.
      if (RULES_BUTTON.SELECTED(x, y)) { 
         RULES_BUTTON.CLICK();
         if (rules == 0) {
            RULES_BUTTON.DRAW_HILITE(0.0, 0.7, 0.0);
            rules = 1;
            DISPLAY_RULES();
         } else {
            RULES_BUTTON.DRAW();
            rules = 0;
            DRAW_GAME();
         }
         return;
      }
      // Start button.
      if (START_BUTTON.SELECTED(x, y) && start == 0) { 
         START_BUTTON.CLICK();
         turn  = blue;
         start = 1;
         START_BUTTON.DRAW_HILITE(0.0, 0.7, 0.0);
         DRAW_MENU();
         DRAW_GAME();
         return;
      }
      //About Button
      if (ABOUT_BUTTON.SELECTED(x, y)) { 
         ABOUT_BUTTON.CLICK();
         if (about == 0) {
            ABOUT_BUTTON.DRAW_HILITE(0.0, 0.7, 0.0);
            about = 1;
            DISPLAY_ABOUT();
         } else {
            ABOUT_BUTTON.DRAW();
            about = 0;
            DRAW_GAME();
         }
         return;
      }
      // Exit button.
      if (EXIT_BUTTON.SELECTED(x, y)) { 
         EXIT_BUTTON.CLICK();
         glDeleteTextures(1, &FeltMap);
         glDeleteTextures(1, &WoodMap);
         glDeleteTextures(1, &RedMap);
         glDeleteTextures(1, &BlueMap);
         glDeleteTextures(1, &TieMap);
         glutDestroyWindow(window);
         exit(0);                   
      }
   }
}

void myMouse1(int button, int state, int x, int y) {
int i;
   glutSetWindow(vuport);
   y = (winHeight - y);
   /*********************/	
   /* Shooter controls.	*/
   /*********************/	
   // Trap mouse button down.
   if (start == 1 && button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
      glFinish();
      mouseX1 = x;
      mouseY1 = y-18;
      // Test for right colored marble.
      gROK = -1;
      gBOK = -1;
      for (i = 0; i < 7; i++) {
          if (turn == red  && MARBLE_SELECTED(Red[i],  mouseX1, mouseY1)) gROK = i;
          if (turn == blue && MARBLE_SELECTED(Blue[i], mouseX1, mouseY1)) gBOK = i;
      }
      if ((gROK != -1) || (gBOK != -1)) {
         start = 2;
         glutSetCursor(GLUT_CURSOR_CROSSHAIR);
      }
   }
   // Trap mouse button up.
   if (start == 2 && button == GLUT_LEFT_BUTTON && state == GLUT_UP) {
      glFinish();
      mouseX2 = x;
      mouseY2 = y-18;
      start = 0;
      glutSetCursor(GLUT_CURSOR_INFO);
      // Get direction and velocity.
      GET_INITIAL_VELOCITY(mouseX1, mouseX2, mouseY1, mouseY2);
      GAME_ANIMATE();
      start = 1;
   }
   // Restart game.
   if (restart) myInit();
}

/****************************************/
/* Resizes screen (if required)         */
/****************************************/
void myResize0(int width, int height) {
   // Define main window.
   glutSetWindow(window);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)winWidth, 0.0, (GLdouble)winHeight, -100.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity(); 
}

void myResize1(int width, int height) {
   // Define viewport.
   glutSetWindow(vuport);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)vupWidth, 0.0, (GLdouble)vupHeight, -5000.0, 5000.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity(); 
}

/****************************************/
/* Redisplays screen (if required)      */
/****************************************/
void myDisplay0(void) {
   DRAW_MENU();
}

void myDisplay1(void) {
   DRAW_GAME();
   GAME_ANIMATE();
}

/****************************************/
/* Initializes OPENGL parameters    	*/
/****************************************/
void myInit(void) {
   restart = false;
   glPointSize(1);
   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClearDepth(5000.0);
   // Define main window.
   glutSetWindow(window);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)winWidth, 0.0, (GLdouble)winHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   // Define viewport.
   glutSetWindow(vuport);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)vupWidth, 0.0, (GLdouble)vupHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   // Seed randome number generator.
   srand(time(NULL));
   CREATE_MARBLES();
   // Enable texture models.
   glEnable(GL_CULL_FACE); 
   TEXTURE_MODEL();
}

/************************************************************/
/* MAIN.                                                    */
/************************************************************/
int main(int argc, char **argv) {
   glutInit(&argc, argv) ;
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);  
   glutInitWindowSize(winWidth, winHeight);
   glutInitWindowPosition(winXpos, winYpos);
   window = glutCreateWindow("Marbles");
   vuport = glutCreateSubWindow(window, vupXpos, vupYpos, vupWidth, vupHeight);
   // Define callback functions.
   glutSetWindow(window);
   glutDisplayFunc(myDisplay0);
   glutReshapeFunc(myResize0);
   glutMouseFunc(myMouse0);
   glutSetWindow(vuport);
   glutDisplayFunc(myDisplay1);
   glutReshapeFunc(myResize1);
   glutMouseFunc(myMouse1);
   // Initialize parameters.
   myInit();
   // Start loop.
   glutMainLoop();
   return 1;
}
